---
title: llarp/rpc/rpc_request.hpp

---

# llarp/rpc/rpc_request.hpp



## Namespaces

| Name           |
| -------------- |
| **[llarp](Namespaces/namespacellarp.md)** <br>[crypto.hpp] |
| **[llarp::rpc](Namespaces/namespacellarp_1_1rpc.md)**  |

## Classes

|                | Name           |
| -------------- | -------------- |
| struct | **[llarp::rpc::rpc_request](Classes/structllarp_1_1rpc_1_1rpc__request.md)**  |




## Source code

```cpp
#pragma once

#include <string_view>
#include <llarp/config/config.hpp>
#include <oxenmq/oxenmq.h>
#include <oxenmq/address.h>
#include <oxen/log/omq_logger.hpp>


namespace llarp::rpc {

  using nlohmann::json;

  using rpc_input = std::variant<std::monostate, nlohmann::json, oxenc::bt_dict_consumer, std::string>;

  struct rpc_request {
    std::variant<std::monostate, std::string, std::string_view, nlohmann::json> body;

    //  Returns string_view of the body contained in rpc_request
    //    If body is empty (std::monostate), returns std::nullopt
    std::optional<std::string_view> view_body() const {
      if (auto* sv = std::get_if<std::string_view>(&body)) return *sv;
      if (auto* s = std::get_if<std::string>(&body)) return *s;
      return std::nullopt;
    }
  };

  template <typename RPC, typename RPCServer, typename RPCCallback>
  auto make_invoke() {
    return [](rpc_request&& request, RPCServer& server) -> typename RPCCallback::result_type {
      RPC rpc{};    // endpoint struct

      // parsing input as bt or json
      //    hand off to parse_request (overloaded versions)
      try {
        if (auto body = request.view_body()) {
          if (body->front() == 'd') { // Looks like a bt dict
            rpc.set_bt();
            parse_request(rpc, oxenc::bt_dict_consumer{*body});
          }
          else
            parse_request(rpc, json::parse(*body));
        } else if (auto* j = std::get_if<json>(&request.body)) {
          parse_request(rpc, std::move(*j));
        } else {
          assert(std::holds_alternative<std::monostate>(request.body));
          parse_request(rpc, std::monostate{});
        }
      } catch (const std::exception& e) {
        throw parse_error{"Failed to parse request parameters: "s + e.what()};
      }

      server.invoke(rpc, std::move(request.context));

      if (rpc.response.is_null())
        rpc.response = json::object();
      
      if (rpc.is_bt())
        return json_to_bt(std::move(rpc.response));
      else
        return std::move(rpc.response);
    }
  }


} // namespace llarp::rpc
```


-------------------------------

Updated on 2023-01-13 at 21:38:12 +0000
