---
title: llarp/rpc/rpc_server.hpp

---

# llarp/rpc/rpc_server.hpp



## Namespaces

| Name           |
| -------------- |
| **[llarp](Namespaces/namespacellarp.md)** <br>[crypto.hpp] |
| **[llarp::rpc](Namespaces/namespacellarp_1_1rpc.md)**  |

## Classes

|                | Name           |
| -------------- | -------------- |
| struct | **[llarp::rpc::rpc_callback](Classes/structllarp_1_1rpc_1_1rpc__callback.md)**  |
| class | **[llarp::rpc::MessageHandler](Classes/classllarp_1_1rpc_1_1MessageHandler.md)**  |
| class | **[llarp::rpc::RPCServer](Classes/classllarp_1_1rpc_1_1RPCServer.md)**  |




## Source code

```cpp
#pragma once

#include "rpc_request_definitions.hpp"
#include "rpc_request_decorators.hpp"
#include <string_view>
#include <llarp/rpc/endpoints.hpp>
#include <llarp/config/config.hpp>
#include <oxenmq/oxenmq.h>
#include <oxenmq/address.h>
#include <oxen/log/omq_logger.hpp>


namespace llarp
{
  struct AbstractRouter;
} 


namespace llarp::rpc
{
  using LMQ_ptr = std::shared_ptr<oxenmq::OxenMQ>;

  class RPCServer;

  //  Stores RPC request callback
  struct rpc_callback {
    using result_type = std::variant<oxenc::bt_value, nlohmann::json, std::string>;
    //  calls with incoming request data; returns response body or throws exception
    result_type(*invoke)(rpc_request&&, RPCServer&);
  };

  //  RPC request registration
  //    Stores references to RPC requests in a unordered map for ease of reference
  //    when adding to server. To add endpoints, define in rpc_request_definitions.hpp
  //    and register in rpc_server.cpp
  extern const std::unordered_map<std::string, std::shared_ptr<const rpc_callback>> rpc_request_map;


  class MessageHandler
  {
  public:
    // constructors + destructor
    explicit MessageHandler(AbstractRouter&);

    explicit MessageHandler(AbstractRouter& _r) : m_Router(_r) {};
    
    ~MessageHandler() = default;

    void 
    SendImmediateResponse(oxenmq::Message& msg);
    void 
    SendDelayedResponse(oxenmq::Message& msg);

    void
    SendImmediateError(oxenmq::Message& msg);

    // member attributes
    AbstractRouter& const m_Router;

    std::unique_ptr<endpoint::Endpoint> endpoint;
  };

  class RPCServer
  {
  public:
    explicit RPCServer(LMQ_ptr, AbstractRouter&);
    ~RPCServer() = default;

    void HandleLogsSubRequest(oxenmq::Message& m);

    void invoke(Halt& halt, rpc_request request);
    void invoke(Version& version, rpc_request request);
    void invoke(Status& status, rpc_request request);
    void invoke(GetStatus& getstatus, rpc_request request);
    void invoke(QuicConnect& quicconnect, rpc_request request);
    void invoke(QuicListener& quiclistener, rpc_request request);
    void invoke(LookupSnode& lookupsnode, rpc_request request);
    void invoke(Exit& exit, rpc_request request);
    void invoke(DNSQuery& dnsquery, rpc_request request);
    void invoke(Config& config, rpc_request request);

    template <typename Result_t>
    std::string
    CreateJSONResponse(Result_t result) 
        { return nlohmann::json{{"error", nullptr},{"result"}, result}.dump(); }

    std::string
    CreateJSONError(std::string_view msg) 
        { return nlohmann::json{{"error", msg}}.dump(); }

    void
    AddCategories();

  private:

    LMQ_ptr m_LMQ;
    AbstractRouter& const m_Router;
    oxen::log::PubsubLogger log_subs;
    std::unique_ptr<MessageHandler> handler;

  };

}  // namespace llarp::rpc
```


-------------------------------

Updated on 2023-01-13 at 21:38:12 +0000
